@@ -610,8 +610,8 @@ struct mach_header_64 {
 };
 
 struct load_command {
-	uint32_t cmd;
-	uint32_t cmdsize;
+	ut32 cmd;
+	ut32 cmdsize;
 };
 
 struct segment_command {
@@ -1433,35 +1433,26 @@ enum X86ThreadFlavors {
 #define EXPORT_SYMBOL_FLAGS_STUB_AND_RESOLVER 0x10
 
 struct dyld_chained_fixups_header {
-	uint32_t fixups_version;
-	uint32_t starts_offset;
-	uint32_t imports_offset;
-	uint32_t symbols_offset;
-	uint32_t imports_count;
-	uint32_t imports_format;
-	uint32_t symbols_format;
+	ut32 fixups_version;
+	ut32 starts_offset;
+	ut32 imports_offset;
+	ut32 symbols_offset;
+	ut32 imports_count;
+	ut32 imports_format;
+	ut32 symbols_format;
 };
 
 struct dyld_chained_starts_in_image {
 	uint32_t seg_count;
 };
 
-struct dyld_chained_starts_in_segment {
-	uint32_t size;
-	uint16_t page_size;
-	uint16_t pointer_format;
-	uint64_t segment_offset;
-	uint32_t max_valid_pointer;
-	uint16_t page_count;
-};
-
 struct rz_dyld_chained_starts_in_segment {
-	uint32_t size;
-	uint16_t page_size;
-	uint16_t pointer_format;
-	uint64_t segment_offset;
-	uint32_t max_valid_pointer;
-	uint16_t page_count;
+	ut32 size;
+	ut16 page_size;
+	ut16 pointer_format;
+	ut64 segment_offset;
+	ut32 max_valid_pointer;
+	ut16 page_count;
 	ut16 *page_start;
 };
 
@@ -1483,6 +1474,14 @@ enum {
 	DYLD_CHAINED_PTR_ARM64E_USERLAND24 = 12,
 };
 
+#include <rz_util/rz_num.h>
+
+#define READ_BITS(dst, count) \
+	do { \
+		dst = raw_val & rz_num_bitmask(count); \
+		raw_val >>= count; \
+	} while (0);
+
 struct dyld_chained_ptr_arm64e_rebase {
 	uint64_t target : 43,
 		high8 : 8,
@@ -1491,6 +1490,14 @@ struct dyld_chained_ptr_arm64e_rebase {
 		auth : 1; // == 0
 };
 
+static inline void dyld_chained_ptr_arm64e_rebase_read(struct dyld_chained_ptr_arm64e_rebase *dst, ut64 raw_val) {
+	READ_BITS(dst->target, 43);
+	READ_BITS(dst->high8, 8);
+	READ_BITS(dst->next, 11);
+	READ_BITS(dst->bind, 1);
+	READ_BITS(dst->auth, 1);
+}
+
 struct dyld_chained_ptr_arm64e_bind {
 	uint64_t ordinal : 16,
 		zero : 16,
@@ -1500,6 +1507,15 @@ struct dyld_chained_ptr_arm64e_bind {
 		auth : 1; // == 0
 };
 
+static inline void dyld_chained_ptr_arm64e_bind_read(struct dyld_chained_ptr_arm64e_bind *dst, ut64 raw_val) {
+	READ_BITS(dst->ordinal, 16);
+	READ_BITS(dst->zero, 16);
+	READ_BITS(dst->addend, 19);
+	READ_BITS(dst->next, 11);
+	READ_BITS(dst->bind, 1);
+	READ_BITS(dst->auth, 1);
+}
+
 struct dyld_chained_ptr_arm64e_auth_rebase {
 	uint64_t target : 32,
 		diversity : 16,
@@ -1510,6 +1526,16 @@ struct dyld_chained_ptr_arm64e_auth_rebase {
 		auth : 1; // == 1
 };
 
+static inline void dyld_chained_ptr_arm64e_auth_rebase_read(struct dyld_chained_ptr_arm64e_auth_rebase *dst, ut64 raw_val) {
+	READ_BITS(dst->target, 32);
+	READ_BITS(dst->diversity, 16);
+	READ_BITS(dst->addrDiv, 1);
+	READ_BITS(dst->key, 2);
+	READ_BITS(dst->next, 11);
+	READ_BITS(dst->bind, 1);
+	READ_BITS(dst->auth, 1);
+}
+
 struct dyld_chained_ptr_arm64e_auth_bind {
 	uint64_t ordinal : 16,
 		zero : 16,
@@ -1521,6 +1547,17 @@ struct dyld_chained_ptr_arm64e_auth_bind {
 		auth : 1; // == 1
 };
 
+static inline void dyld_chained_ptr_arm64e_auth_bind_read(struct dyld_chained_ptr_arm64e_auth_bind *dst, ut64 raw_val) {
+	READ_BITS(dst->ordinal, 16);
+	READ_BITS(dst->zero, 16);
+	READ_BITS(dst->diversity, 16);
+	READ_BITS(dst->addrDiv, 1);
+	READ_BITS(dst->key, 2);
+	READ_BITS(dst->next, 11);
+	READ_BITS(dst->bind, 1);
+	READ_BITS(dst->auth, 1);
+}
+
 struct dyld_chained_ptr_64_rebase {
 	uint64_t target : 36,
 		high8 : 8,
@@ -1529,6 +1566,14 @@ struct dyld_chained_ptr_64_rebase {
 		bind : 1; // == 0
 };
 
+static inline void dyld_chained_ptr_64_rebase_read(struct dyld_chained_ptr_64_rebase *dst, ut64 raw_val) {
+	READ_BITS(dst->target, 36);
+	READ_BITS(dst->high8, 8);
+	READ_BITS(dst->reserved, 7);
+	READ_BITS(dst->next, 12);
+	READ_BITS(dst->bind, 1);
+}
+
 struct dyld_chained_ptr_64_bind {
 	uint64_t ordinal : 24,
 		addend : 8,
@@ -1537,6 +1582,14 @@ struct dyld_chained_ptr_64_bind {
 		bind : 1; // == 1
 };
 
+static inline void dyld_chained_ptr_64_bind_read(struct dyld_chained_ptr_64_bind *dst, ut64 raw_val) {
+	READ_BITS(dst->ordinal, 24);
+	READ_BITS(dst->addend, 8);
+	READ_BITS(dst->reserved, 19);
+	READ_BITS(dst->next, 12);
+	READ_BITS(dst->bind, 1);
+}
+
 /* WARNING: this is guesswork based on trial and error */
 struct dyld_chained_ptr_arm64e_cache_rebase {
 	uint64_t target : 43,
@@ -1545,6 +1598,13 @@ struct dyld_chained_ptr_arm64e_cache_rebase {
 		auth : 1; // == 0
 };
 
+static inline void dyld_chained_ptr_arm64e_cache_rebase_read(struct dyld_chained_ptr_arm64e_cache_rebase *dst, ut64 raw_val) {
+	READ_BITS(dst->target, 43);
+	READ_BITS(dst->high8, 8);
+	READ_BITS(dst->next, 12);
+	READ_BITS(dst->auth, 1);
+}
+
 struct dyld_chained_ptr_arm64e_cache_auth_rebase {
 	uint64_t target : 32,
 		diversity : 16,
@@ -1554,6 +1614,15 @@ struct dyld_chained_ptr_arm64e_cache_auth_rebase {
 		auth : 1; // == 1
 };
 
+static inline void dyld_chained_ptr_arm64e_cache_auth_rebase_read(struct dyld_chained_ptr_arm64e_cache_auth_rebase *dst, ut64 raw_val) {
+	READ_BITS(dst->target, 32);
+	READ_BITS(dst->diversity, 16);
+	READ_BITS(dst->addrDiv, 1);
+	READ_BITS(dst->key, 2);
+	READ_BITS(dst->next, 12);
+	READ_BITS(dst->auth, 1);
+}
+
 struct dyld_chained_ptr_arm64e_bind24 {
 	uint64_t ordinal : 24,
 		zero : 8,
@@ -1563,6 +1632,15 @@ struct dyld_chained_ptr_arm64e_bind24 {
 		auth : 1; // == 0
 };
 
+static inline void dyld_chained_ptr_arm64e_bind24_read(struct dyld_chained_ptr_arm64e_bind24 *dst, ut64 raw_val) {
+	READ_BITS(dst->ordinal, 24);
+	READ_BITS(dst->zero, 8);
+	READ_BITS(dst->addend, 19);
+	READ_BITS(dst->next, 11);
+	READ_BITS(dst->bind, 1);
+	READ_BITS(dst->auth, 1);
+}
+
 struct dyld_chained_ptr_arm64e_auth_bind24 {
 	uint64_t ordinal : 24,
 		zero : 8,
@@ -1574,4 +1652,19 @@ struct dyld_chained_ptr_arm64e_auth_bind24 {
 		auth : 1; // == 1
 };
 
+static inline void dyld_chained_ptr_arm64e_auth_bind24_read(struct dyld_chained_ptr_arm64e_auth_bind24 *dst, ut64 raw_val) {
+	READ_BITS(dst->ordinal, 24);
+	READ_BITS(dst->zero, 8);
+	READ_BITS(dst->diversity, 16);
+	READ_BITS(dst->addrDiv, 1);
+	READ_BITS(dst->key, 2);
+	READ_BITS(dst->next, 11);
+	READ_BITS(dst->bind, 1);
+	READ_BITS(dst->auth, 1);
+}
+
+// When adding more structs/readers here, also add tests to test/unit/test_bin_mach0.c!
+
+#undef READ_BITS
+
 #endif
