@@ -4,6 +4,34 @@
 #define RZ_BIN_PE64 1
 #include "bin_pe.inc"
 
+// equivalent of PE64_UNWIND_INFO but endian-free
+typedef struct windows_x64_unwind_info_t {
+	ut8 Version; // : 3;
+	ut8 Flags; // : 5;
+	ut8 SizeOfProlog;
+	ut8 CountOfCodes;
+	ut8 FrameRegister; // : 4;
+	ut8 FrameOffset; // : 4;
+} WinUnwindInfo;
+
+static bool windows_unwind_info_read(WinUnwindInfo *info, RzBuffer *b, ut64 at) {
+	ut8 tmp[sizeof(PE64_UNWIND_INFO)] = { 0 };
+
+	if (!rz_buf_read_at(b, at, tmp, sizeof(tmp))) {
+		return false;
+	}
+
+	// The ordering of bits in C bitfields is implementation defined.
+	// this ensures the endianness (here is little endian) is kept
+	info->Version = tmp[0] & 0x07;
+	info->Flags = tmp[0] >> 3;
+	info->SizeOfProlog = tmp[1];
+	info->CountOfCodes = tmp[2];
+	info->FrameRegister = tmp[3] & 0x0F;
+	info->FrameOffset = tmp[3] >> 4;
+	return true;
+}
+
 static bool check_buffer(RzBuffer *b) {
 	ut64 length = rz_buf_size(b);
 	if (length <= 0x3d) {
@@ -34,7 +62,7 @@ static bool check_buffer(RzBuffer *b) {
 	return false;
 }
 
-static RzList *fields(RzBinFile *bf) {
+static RzList /*<RzBinField *>*/ *fields(RzBinFile *bf) {
 	RzList *ret = rz_list_newf((RzListFree)rz_bin_field_free);
 	if (!ret) {
 		return NULL;
@@ -358,70 +386,146 @@ static void header(RzBinFile *bf) {
 	}
 }
 
-static RzList *trycatch(RzBinFile *bf) {
-	RzIO *io = bf->rbin->iob.io;
+#define RzBinPEObj struct PE_(rz_bin_pe_obj_t)
+static inline int find_section(RzBinPEObj *bin, PE_DWord rva) {
+	PE_DWord section_base;
+	int i, section_size;
+	for (i = 0; i < bin->num_sections; i++) {
+		section_base = bin->sections[i].vaddr;
+		section_size = bin->sections[i].vsize;
+		if (rva >= section_base && rva < section_base + section_size) {
+			return i;
+		}
+	}
+	return -1;
+}
+
+static inline ut64 rva_to_paddr(const struct rz_bin_pe_section_t *section, ut64 rva) {
+	return section->paddr + (rva - section->vaddr);
+}
+
+static inline const struct rz_bin_pe_section_t *get_section(RzBinPEObj *bin, const struct rz_bin_pe_section_t *unwind_data_section, PE_DWord rva) {
+	if (!unwind_data_section || (rva < unwind_data_section->vaddr || rva > unwind_data_section->vaddr + unwind_data_section->vsize)) {
+		const int section_idx = find_section(bin, rva);
+		if (section_idx == -1) {
+			return NULL;
+		}
+		if (bin->sections[section_idx].paddr > rz_buf_size(bin->b)) {
+			return NULL;
+		}
+		return &bin->sections[section_idx];
+	}
+	return unwind_data_section;
+}
+
+static bool read_pe64_runtime_function(RzBuffer *buf, ut64 base, PE64_RUNTIME_FUNCTION *rfcn, bool big_endian) {
+	ut64 offset = base;
+	return rz_buf_read_ble32_offset(buf, &offset, &rfcn->BeginAddress, big_endian) &&
+		rz_buf_read_ble32_offset(buf, &offset, &rfcn->EndAddress, big_endian) &&
+		rz_buf_read_ble32_offset(buf, &offset, &rfcn->UnwindInfoAddress, big_endian);
+}
+
+static bool read_pe64_scope_record(RzBuffer *buf, ut64 base, PE64_SCOPE_RECORD *record, bool big_endian) {
+	ut64 offset = base;
+	return rz_buf_read_ble32_offset(buf, &offset, &record->BeginAddress, big_endian) &&
+		rz_buf_read_ble32_offset(buf, &offset, &record->EndAddress, big_endian) &&
+		rz_buf_read_ble32_offset(buf, &offset, &record->HandlerAddress, big_endian) &&
+		rz_buf_read_ble32_offset(buf, &offset, &record->JumpTarget, big_endian);
+}
+
+static RzList /*<RzBinTrycatch *>*/ *trycatch(RzBinFile *bf) {
 	ut64 baseAddr = bf->o->opts.baseaddr;
-	int i;
 	ut64 offset;
-	ut32 c_handler = 0;
 
 	struct PE_(rz_bin_pe_obj_t) *bin = bf->o->bin_obj;
+	if (bin->optional_header->NumberOfRvaAndSizes < PE_IMAGE_DIRECTORY_ENTRY_EXCEPTION) {
+		return NULL;
+	}
+	if (bin->optional_header->DataDirectory[PE_IMAGE_DIRECTORY_ENTRY_EXCEPTION].VirtualAddress == 0) {
+		return NULL;
+	}
 	PE_(image_data_directory) *expdir = &bin->optional_header->DataDirectory[PE_IMAGE_DIRECTORY_ENTRY_EXCEPTION];
 	if (!expdir->Size) {
 		return NULL;
 	}
+	const struct rz_bin_pe_section_t *pdata = get_section(bin, NULL, expdir->VirtualAddress);
+	if (!pdata) {
+		return NULL;
+	}
+	const ut64 paddr = rva_to_paddr(pdata, expdir->VirtualAddress);
+	if (paddr > rz_buf_size(bin->b)) {
+		return NULL;
+	}
 
 	RzList *tclist = rz_list_newf((RzListFree)rz_bin_trycatch_free);
 	if (!tclist) {
 		return NULL;
 	}
+	const struct rz_bin_pe_section_t *unwind_data_section = NULL;
 
-	for (offset = expdir->VirtualAddress; offset < (ut64)expdir->VirtualAddress + expdir->Size; offset += sizeof(PE64_RUNTIME_FUNCTION)) {
-		PE64_RUNTIME_FUNCTION rfcn;
-		bool suc = rz_io_read_at_mapped(io, offset + baseAddr, (ut8 *)&rfcn, sizeof(rfcn));
+	const ut64 end = RZ_MIN(rz_buf_size(bin->b), pdata->paddr + pdata->size);
+
+	for (offset = paddr; offset < end; offset += sizeof(PE64_RUNTIME_FUNCTION)) {
+		PE64_RUNTIME_FUNCTION rfcn = { 0 };
+		bool suc = read_pe64_runtime_function(bin->b, offset, &rfcn, bin->big_endian);
 		if (!rfcn.BeginAddress) {
 			break;
 		}
 		ut32 savedBeginOff = rfcn.BeginAddress;
 		ut32 savedEndOff = rfcn.EndAddress;
 		while (suc && rfcn.UnwindData & 1) {
-			suc = rz_io_read_at_mapped(io, baseAddr + (rfcn.UnwindData & ~1), (ut8 *)&rfcn, sizeof(rfcn));
+			ut64 paddr = rva_to_paddr(pdata, rfcn.UnwindData & ~1);
+			suc = read_pe64_runtime_function(bin->b, paddr, &rfcn, bin->big_endian);
 		}
 		rfcn.BeginAddress = savedBeginOff;
 		rfcn.EndAddress = savedEndOff;
 		if (!suc) {
 			continue;
 		}
-		PE64_UNWIND_INFO info;
-		suc = rz_io_read_at_mapped(io, rfcn.UnwindData + baseAddr, (ut8 *)&info, sizeof(info));
+		unwind_data_section = get_section(bin, unwind_data_section, rfcn.UnwindData);
+		if (!unwind_data_section) {
+			continue;
+		}
+		ut64 unwind_data_paddr = rva_to_paddr(unwind_data_section, rfcn.UnwindData);
+		if (unwind_data_paddr > unwind_data_section->paddr + unwind_data_section->size) {
+			continue;
+		}
+		WinUnwindInfo info;
+		suc = windows_unwind_info_read(&info, bin->b, unwind_data_paddr);
 		if (!suc || info.Version != 1 || (!(info.Flags & PE64_UNW_FLAG_EHANDLER) && !(info.Flags & PE64_UNW_FLAG_CHAININFO))) {
 			continue;
 		}
 
 		ut32 sizeOfCodeEntries = info.CountOfCodes % 2 ? info.CountOfCodes + 1 : info.CountOfCodes;
 		sizeOfCodeEntries *= sizeof(PE64_UNWIND_CODE);
-		ut64 exceptionDataOff = baseAddr + rfcn.UnwindData + offsetof(PE64_UNWIND_INFO, UnwindCode) + sizeOfCodeEntries;
+		ut64 exceptionDataOff = unwind_data_paddr + offsetof(PE64_UNWIND_INFO, UnwindCode) + sizeOfCodeEntries;
 
 		if (info.Flags & PE64_UNW_FLAG_CHAININFO) {
 			savedBeginOff = rfcn.BeginAddress;
 			savedEndOff = rfcn.EndAddress;
 			do {
-				if (!rz_io_read_at_mapped(io, exceptionDataOff, (ut8 *)&rfcn, sizeof(rfcn))) {
+				if (!read_pe64_runtime_function(bin->b, exceptionDataOff, &rfcn, bin->big_endian)) {
 					break;
 				}
-				suc = rz_io_read_at_mapped(io, rfcn.UnwindData + baseAddr, (ut8 *)&info, sizeof(info));
+				unwind_data_section = get_section(bin, unwind_data_section, rfcn.UnwindData);
+				if (!unwind_data_section) {
+					break;
+				}
+				unwind_data_paddr = rva_to_paddr(unwind_data_section, rfcn.UnwindData);
+				suc = windows_unwind_info_read(&info, bin->b, unwind_data_paddr);
 				if (!suc || info.Version != 1) {
 					break;
 				}
-				while (suc && (rfcn.UnwindData & 1)) {
-					suc = rz_io_read_at_mapped(io, baseAddr + (rfcn.UnwindData & ~1), (ut8 *)&rfcn, sizeof(rfcn));
+				while (suc && (unwind_data_paddr & 1)) {
+					suc = read_pe64_runtime_function(bin->b, unwind_data_paddr & ~1, &rfcn, bin->big_endian);
+					unwind_data_paddr = rva_to_paddr(unwind_data_section, rfcn.UnwindData);
 				}
 				if (!suc || info.Version != 1) {
 					break;
 				}
 				sizeOfCodeEntries = info.CountOfCodes % 2 ? info.CountOfCodes + 1 : info.CountOfCodes;
 				sizeOfCodeEntries *= sizeof(PE64_UNWIND_CODE);
-				exceptionDataOff = baseAddr + rfcn.UnwindData + offsetof(PE64_UNWIND_INFO, UnwindCode) + sizeOfCodeEntries;
+				exceptionDataOff = unwind_data_paddr + offsetof(PE64_UNWIND_INFO, UnwindCode) + sizeOfCodeEntries;
 			} while (info.Flags & PE64_UNW_FLAG_CHAININFO);
 			if (!(info.Flags & PE64_UNW_FLAG_EHANDLER)) {
 				continue;
@@ -430,35 +534,22 @@ static RzList *trycatch(RzBinFile *bf) {
 			rfcn.EndAddress = savedEndOff;
 		}
 
-		ut32 handler;
-		if (!rz_io_read_at_mapped(io, exceptionDataOff, (ut8 *)&handler, sizeof(handler))) {
-			continue;
-		}
-		if (c_handler && c_handler != handler) {
+		ut32 handler = 0;
+		if (!rz_buf_read_ble32_at(bin->b, exceptionDataOff, &handler, bin->big_endian)) {
 			continue;
 		}
 		exceptionDataOff += sizeof(ut32);
 
-		if (!c_handler) {
-			ut32 magic, rva_to_fcninfo;
-			if (rz_io_read_at_mapped(io, exceptionDataOff, (ut8 *)&rva_to_fcninfo, sizeof(rva_to_fcninfo)) &&
-				rz_io_read_at_mapped(io, baseAddr + rva_to_fcninfo, (ut8 *)&magic, sizeof(magic))) {
-				if (magic >= 0x19930520 && magic <= 0x19930522) {
-					// __CxxFrameHandler3 or __GSHandlerCheck_EH
-					continue;
-				}
-			}
-		}
-
 		PE64_SCOPE_TABLE tbl;
-		if (!rz_io_read_at_mapped(io, exceptionDataOff, (ut8 *)&tbl, sizeof(tbl))) {
+		if (!rz_buf_read_ble32_at(bin->b, exceptionDataOff, &tbl.Count, bin->big_endian)) {
 			continue;
 		}
-
+		const ut64 last_scope_addr = RZ_MIN(rz_buf_size(bin->b), unwind_data_section->paddr + unwind_data_section->size) - sizeof(PE64_SCOPE_RECORD);
 		PE64_SCOPE_RECORD scope;
 		ut64 scopeRecOff = exceptionDataOff + sizeof(tbl);
-		for (i = 0; i < tbl.Count; i++) {
-			if (!rz_io_read_at_mapped(io, scopeRecOff, (ut8 *)&scope, sizeof(PE64_SCOPE_RECORD))) {
+		int i;
+		for (i = 0; i < tbl.Count && scopeRecOff <= last_scope_addr; i++, scopeRecOff += sizeof(PE64_SCOPE_RECORD)) {
+			if (!read_pe64_scope_record(bin->b, scopeRecOff, &scope, bin->big_endian)) {
 				break;
 			}
 			if (scope.BeginAddress > scope.EndAddress || scope.BeginAddress == UT32_MAX || scope.EndAddress == UT32_MAX || !scope.BeginAddress || !scope.EndAddress) {
@@ -469,7 +560,8 @@ static RzList *trycatch(RzBinFile *bf) {
 			}
 			if (!scope.JumpTarget) {
 				// scope.HandlerAddress == __finally block
-				continue;
+				scope.JumpTarget = scope.HandlerAddress;
+				scope.HandlerAddress = 1;
 			}
 			ut64 handlerAddr = scope.HandlerAddress == 1 ? 0 : scope.HandlerAddress + baseAddr;
 			RzBinTrycatch *tc = rz_bin_trycatch_new(
@@ -478,9 +570,7 @@ static RzList *trycatch(RzBinFile *bf) {
 				scope.EndAddress + baseAddr,
 				scope.JumpTarget + baseAddr,
 				handlerAddr);
-			c_handler = handler;
 			rz_list_append(tclist, tc);
-			scopeRecOff += sizeof(PE64_SCOPE_RECORD);
 		}
 	}
 	return tclist;
@@ -501,14 +591,17 @@ RzBinPlugin rz_bin_plugin_pe64 = {
 	.sections = &sections,
 	.symbols = &symbols,
 	.imports = &imports,
+	.strings = &strings,
 	.info = &info,
 	.header = &header,
 	.fields = &fields,
 	.libs = &libs,
 	.relocs = &relocs,
+	.get_offset = &get_offset,
 	.get_vaddr = &get_vaddr,
 	.trycatch = &trycatch,
 	.hashes = &compute_hashes,
+	.resources = &resources,
 	.section_flag_to_rzlist = &PE_(section_flag_to_rzlist),
 };
 
